'-----------------------------------------------------------------------------
' <summary>
' Copyright Keystroke IT 2003
' <WrittenBy>Minas Casiou, Keystroke IT Australia</WrittenBy>
' </summary>
'-----------------------------------------------------------------------------
Option Explicit On

Imports ESB.Core.Pipeline
Imports ESB.Core.XMLSchemaClasses
Imports System.Reflection


''' <summary>
''' This class is an ESB.NET service handler that enables services to have their own ESB.NET Service Pipeline
''' deployed in a folder related to the service.
''' This then allows for these services to be first class citizens of ESB.NET. 
'''  - Exposes ESB.NET feature available to fully configured services
'''  - Allows for a simple configuration per service, that lives with the service
'''  - Removes the dependency on the centralized ServicePipeline.config file
''' 
''' The file MUST be in a directory that follows a particular convention. 
''' The local service config file itself does not have to be in the same directory as the service handlers.
''' This is because there will often be more than one service handler for a service.
''' </summary>
''' <remarks>
''' 1 - To be configured in the HorizontalPre of the ServicePipeline
''' 2 - Works as follows:
'''   2.1 - Looks at the service name for a given request
'''   2.2 - Looks for a local configuration file in the service directory (LobSystems\?)
'''         using the convention for locating local Pipeline Config files
'''   2.3.1 - If Local Service Configuration File *DOES EXIST*:
'''             ==>Injects the pipeline configuration from the file into the current service pipeline (into the Vertical section)
'''   2.3.2 - If Local Service Configuration File *DOES NOT EXIST*:
'''             ==>DO NOTHING
'''   2.4 - *ALWAYS* Removes itself from the current service pipeline (out of the HorizontalPre, so that it does not get executed again)
'''   2.5 - Stores the updated pipeline to the cache. As such, stopping this handler from being executed a second time.
''' 
''' 
''' 3 - Other 
'''   3.1 - What about HorizontalPre/Post, on a per locally configured service basis...??? - Leave till later.
'''   3.2 - When PipelineID is specified, also needs to qry same LCS config file and inject appropriate entry
'''   3.3 - Needs different directory (slightly mod'd convention) for when a service request does not contain a Context.
'''         i.e. for Pub-Sub scenarios where just a document (ResolveName) is specified
'''         Use a different directory in this case.
'''         eg.
'''            LobSystems\lcs
'''                      \ctx  (when Context is specified)
'''                          \MyFirstLcsService\Config\Lcs.MyFirstLcsService.config
'''                 OR
'''                      \rn   (when only Resolve Name is specified - i.e. No Context), or neither ResolveName nor Context are specified (prb. silly scenario).
'''                          \MyFirstLcsServiceWithBlankContext\Config\Lcs.MyFirstLcsServiceWithBlankContext.config
''' 
'''          In Both cases, when the PipelineID is specified, the same file will be queried, but only the single ContextEntry 
'''          for the given PipelineID will be injected as opposed to the whole PipelineEntry.
''' 
''' 4 - Is flushed when the ServicePipeline.config file is updated and the Cached Service Pipelines are flushed from cache.
''' 
''' 5 - Demonstrates the flexibility, and how to build such an adaptor to suit specific needs WITHOUT an ESB.NET core code change.
''' 
''' </remarks>
<System.Runtime.InteropServices.ComVisible(False)> Public Class InjectLocalConfigIntoPipeline
    Inherits ServiceBase

#Region "Constants"
    Private Const STR_ESBInjectLocalConfigurationHandlerAssembliesRootPath As String = "ESB.ConventionOverConfiguration.LocallyConfiguredServiceRootPath"
#End Region

    Dim m_oDocRq As Object
    Dim m_oDocRs As Object

    Dim m_bInit As Boolean
   
    Dim m_bServiceRequiresLocalConfigInjection As Boolean = False
    Dim m_LocallyConfiguredServiceRootPath As String

    Dim m_sServiceName As String
    Dim m_sResolveName As String
    Dim m_sPipelineID As String

    Dim m_oLcsPipelineCfg As ESB.Core.Pipeline.InteropPipelineCfg.PipelineEntry
    'Dim m_oLcsPipelineCfg As ESB.Core.Pipeline.InteropPipelineCfg.Config

    'Service Entry Point 
    Public Overrides Function ProcessMsgSync(ByVal oEnv As IEnvelope) As IEnvelope
        StartMethod()

        Try
            MyBase.ProcessMsgSync(oEnv)
            init()
           
            DoWork()

            Return m_oEnvRs

        Catch ex As Exception
            ExceptionManager.Publish(ex)
            LogError(LogDefs.eLogLevel.eLOG_ERROR, ex.ToString())
            Throw

        Finally
            EndMethod()
        End Try
    End Function

    Protected Sub DoWork()
        If ShouldInjectLocalConfigIntoPipeline() Then
            '3.1 - need to implement...???
        End If
    End Sub

    ''' <summary>
    ''' Determine whether injection processing is required for this request...
    ''' </summary>
    ''' <returns>True if should do config injection processing. False otherwise.</returns>
    ''' <remarks>
    ''' 
    '''   2.1 - Looks at the service name for a given request
    '''   2.2 - Looks for a local configuration file in the service directory (LobSystems\?)
    '''         using the convention for locating local Pipeline Config files
    '''   2.3.1 - If Local Service Configuration File *DOES EXIST*:
    '''             ==>Injects the pipeline configuration from the file into the current service pipeline (into the Vertical section)
    '''             2.3.1.1 - Looks up the current context, gets the PipelineManager and 
    '''             2.3.1.2 - subsequently the current PipelineConfig for this particular request.
    '''   2.3.2 - If Local Service Configuration File *DOES NOT EXIST*:
    '''             ==>DO NOTHING
    '''</remarks>
    Protected Function ShouldInjectLocalConfigIntoPipeline() As Boolean
        m_bServiceRequiresLocalConfigInjection = False

        Dim oServerContextImpl As ESB.Core.Services.Server.ServerContext = GetContext()

        '2.1 - Given Service name, look up stuff...
        m_sServiceName = m_oEnvRq.Context

        '2.1 Cont'd - Should take ResolveName into account as well!
        m_sResolveName = m_oEnvRq.ResolveName

        m_sPipelineID = m_oEnvRq.GetCustomParam(C_sCustomParamESBCore, C_sPipelineManager_PipelineID)

        '2.2 - Look for ServiceConfig in following directory:
        ''  C:\ESB\Dev\Source\ESB\LobSystems\lcs\<ServiceName>.<ResolveName>
        Dim sLcsPathForRequest As String = GetLcsPathForRequest(m_sServiceName, m_sResolveName, m_sPipelineID)

        'if Exists file at path :    sLcsPathForRequest
        ' with any content - parse & attempt to read content...
        ' then return true...

        If My.Computer.FileSystem.FileExists(ESB.Core.Util.ExpandRelativePath(sLcsPathForRequest)) Then
            Dim sLcsConfigData As String = ESB.Core.Util.File.Read(sLcsPathForRequest)
            If sLcsConfigData <> "" Then
                Dim oXmlUtil As New ESB.Core.XMLSchemaClasses.XMLUtil

                'Will throw expection on failure...
                m_oLcsPipelineCfg = oXmlUtil.DeSerialize(sLcsConfigData, GetType(ESB.Core.Pipeline.InteropPipelineCfg.PipelineEntry))
                m_bServiceRequiresLocalConfigInjection = True
            End If
        Else
            m_bServiceRequiresLocalConfigInjection = False
        End If


        '2.3.1
        'For the m_oLcsPipelineCfg above, get the Relevant Entries & push into the Pipeline...
        Dim oCePipelineIDOnly As ESB.Core.Pipeline.InteropPipelineCfg.ContextEntry = Nothing
        If m_bServiceRequiresLocalConfigInjection = True Then
            For Each oContextEntry As ESB.Core.Pipeline.InteropPipelineCfg.ContextEntry In m_oLcsPipelineCfg.ResolveNameEntry(0).ContextEntry
                'a) ensure we're got the correct config...
                If m_oLcsPipelineCfg.ResolveNameEntry(0).ContextEntry(0).Context = m_sServiceName And _
                    m_oLcsPipelineCfg.ResolveNameEntry(0).ContextEntry(0).ResolveName = m_sResolveName Then

                    'Set the ContextEntryType to Vertical so it gets handled properly!!!
                    oContextEntry.eContextEntryType = InteropPipelineCfg.ContextEntryEContextEntryType.eVerticalHandler

                    If m_sPipelineID <> "" Then
                        If m_oLcsPipelineCfg.ResolveNameEntry(0).ContextEntry(0).PipelineID = m_sPipelineID Then
                            oCePipelineIDOnly = oContextEntry
                        End If
                    End If
                End If
                If ShouldLog(LogDefs.eLogLevel.eLOG_TRACE) Then LogMsg(LogDefs.eLogLevel.eLOG_TRACE, "Error:Not Injecting to Pipeline for request:m_sServiceName ==>" & m_sServiceName & "<== m_sResolveName ==>" & m_sResolveName & "<==" & "sLcsPathForRequest==>:" & sLcsPathForRequest & "because ResolveName and Context in Lcs Config file does NOT match that of the request;")
            Next
        End If


        '2.3.1.1
        'Different Pipeline Manager's are implemented differently cos of lack of MI support!!!
        'So, to get to PipelineManagerBase class, we expose a property on each of them, however...
        'This property is not currently available via an interface. So...
        'We know the method's there...Just get to it!!!
        'So...use dynamic features to do so...(i.e. late binding...)
        Dim oPipelineIf As Object = oServerContextImpl.PipelineManager
        Dim oPipelineBaseImpl As Pipeline.PipelineManagerBase = oPipelineIf.PipelineManagerBase

        '2.3.1.2
        Dim oCfg As ESB.Core.Pipeline.InteropPipelineCfg.Config

        '++The cached pipeline for this particular request
        oCfg = oPipelineBaseImpl.m_oPlgRequestPipelineCfgCLR

        'Instruct the PipelineManager to reprocess the Pipeline to take into account the newly added items...
        'It will do this immediately after executing this handler, and before executing any other handlers...
        oPipelineBaseImpl.RestartProcessingPipeline = True

        Dim oCE As InteropPipelineCfg.ContextEntry = Nothing
        'See if there's anything here in the Vertical... 
        'If not, then just add in the above
        'If there is, then ... just add it in anyway...???

        Dim oCeToRemove_Inject As InteropPipelineCfg.ContextEntry  'this is the  InjectLocalConfigIntoPipeline Entry...
        Dim oCeToRemove_Convention As InteropPipelineCfg.ContextEntry  'this is the  InjectLocalConfigIntoPipeline Entry...
        'Look for something in the Vertical without an AlwaysExecute value set...
        'Try
        For Each oCE In oCfg.PipelineConfig(0).Vertical(0).PipelineEntry(0).ResolveNameEntry(0).ContextEntry
            If oCE.eContextEntryType = InteropPipelineCfg.ContextEntryEContextEntryType.eHorizontalPreHandler Then
                'Remove this handler (InjectLocalConfigIntoPipeline)...if there...
                'Store a reference to it so we can remove it once we've exited this loop. 
                'Otherwise we'll get an InvalidOperationException or something...
                If oCE.MultiLevelOverrides.TransactionComponent.Value.Contains("InjectLocalConfigIntoPipeline") Then 'The name of this class - TODO: Change to use reflection etc.
                    oCeToRemove_Inject = oCE
                End If
                Exit For
            End If
        Next

        For Each oCE In oCfg.PipelineConfig(0).Vertical(0).PipelineEntry(0).ResolveNameEntry(0).ContextEntry
            If oCE.eContextEntryType = InteropPipelineCfg.ContextEntryEContextEntryType.eHorizontalPostHandler Then
                'Remove this handler (InvokeViaConvention)...if there...
                'Store a reference to it so we can remove it once we've exited this loop. 
                'Otherwise we'll get an InvalidOperationException or something...
                If oCE.MultiLevelOverrides.TransactionComponent.Value.Contains("InvokeViaConvention") Then 'The name of any of the CoC handler classes - TODO: Change to use reflection etc.
                    oCeToRemove_Convention = oCE
                End If
                Exit For
            End If
        Next

        If m_bServiceRequiresLocalConfigInjection = True Then

            '3.2
            'Add the LocalConfig to the pipeline...
            If Not oCePipelineIDOnly Is Nothing Then
                'Only add the specific PipelineID
                oCfg.PipelineConfig(0).Vertical(0).PipelineEntry(0).ResolveNameEntry(0).ContextEntry.Add(oCePipelineIDOnly)
            Else
                'Add whole config to the pipeline
                oCfg.PipelineConfig(0).Vertical(0).PipelineEntry(0).ResolveNameEntry(0).ContextEntry.AddRange(m_oLcsPipelineCfg.ResolveNameEntry(0).ContextEntry)
            End If

            'Remove the CoC handler, only if we're adding a Config entry that makes the CoC handler unnecessary.
            'This is a performance optimization.
            If Not oCeToRemove_Convention Is Nothing Then
                oCfg.PipelineConfig(0).Vertical(0).PipelineEntry(0).ResolveNameEntry(0).ContextEntry.Remove(oCeToRemove_Convention)
            End If

        End If

        '2.4 - *ALWAYS* Removes itself from the current service pipeline (out of the HorizontalPre, so that it does not get executed again)
        'Remove this handler (InjectLocalConfigIntoPipeline)...
        If Not oCeToRemove_Inject Is Nothing Then
            oCfg.PipelineConfig(0).Vertical(0).PipelineEntry(0).ResolveNameEntry(0).ContextEntry.Remove(oCeToRemove_Inject)
        End If

        '2.5 - Update Pipeline in Cache...
        If oPipelineBaseImpl.SetToCache(m_sResolveName, m_sServiceName & "." & m_sPipelineID, DirectCast(oCfg, Object)) = False Then
            LogError(LogDefs.eLogLevel.eLOG_ERROR, "Error Setting Cache: Resolve Name=" & m_sResolveName & "Context=" & m_sServiceName)
        End If

        'Also need to update the Pipeline Manager's currently executing Pipeline variable!
        oPipelineBaseImpl.m_oPlgRequestPipelineCfgCLR = oCfg

        m_bServiceRequiresLocalConfigInjection = True 'and BTW, it's already been done!
        Return True
        'Catch ex As Exception                                                                            
        '    Return False
        'End Try

    End Function


    ''' <summary>
    ''' 
    ''' 
    ''' </summary>
    ''' <param name="sServiceName"></param>
    ''' <param name="sResolveName"></param>
    ''' <returns></returns>
    ''' <remarks>
    '''  3.2 - When PipelineID is specified, also needs to qry same LCS config file and inject appropriate entry
    '''   3.3 - Needs different directory (slightly mod'd convention) for when a service request does not contain a Context.
    '''         i.e. for Pub-Sub scenarios where just a document (ResolveName) is specified
    '''         Use a different directory in this case.
    '''         eg.
    '''            LobSystems\lcs
    '''                      \ctx  (when Context is specified)
    '''                          \MyFirstLcsService\Config\Lcs.MyFirstLcsService.config
    '''                 OR
    '''                      \rn   (when only Resolve Name is specified - i.e. No Context), or neither ResolveName nor Context are specified (prb. silly scenario).
    '''                          \MyFirstLcsServiceWithBlankContext(ResolveName)\Config\Lcs.MyFirstLcsServiceWithBlankContext(ResolveName).config
    ''' 
    '''          In Both cases, when the PipelineID is specified, the same file will be queried, but only the single ContextEntry 
    '''          for the given PipelineID will be injected as opposed to the whole PipelineEntry.
    ''' </remarks>
    Private Function GetLcsPathForRequest(ByVal sServiceName As String, ByVal sResolveName As String, ByVal sPipelineID As String) As String
        ''  C:\ESB\Dev\Source\ESB\LobSystems\lcs\<ServiceName>.<ResolveName>

        Dim sUrlEncodedServiceName As String = MakeValidFileName(sServiceName)
        Dim sUrlEncodedResolveName As String = MakeValidFileName(sResolveName)

        If sUrlEncodedServiceName = "" And sUrlEncodedResolveName = "" Then
            Return ESB.Core.Util.ExpandRelativePath(m_LocallyConfiguredServiceRootPath & "\rn\Config\Lcs.config")  ' Can only hsve one of these per ESB.NET instance...
        End If

        If sUrlEncodedServiceName = "" And sUrlEncodedResolveName <> "" Then
            Return ESB.Core.Util.ExpandRelativePath(m_LocallyConfiguredServiceRootPath & "\rn\" & sUrlEncodedResolveName & "\Config\Lcs." & sUrlEncodedResolveName & ".config")
        End If

        If sUrlEncodedServiceName <> "" And sUrlEncodedResolveName = "" Then
            Return ESB.Core.Util.ExpandRelativePath(m_LocallyConfiguredServiceRootPath & "\ctx\" & sUrlEncodedServiceName & "\Config\Lcs." & sUrlEncodedServiceName & ".config")
        End If

        If sUrlEncodedServiceName <> "" And sUrlEncodedResolveName <> "" Then
            'i.e. same as above - DO NOT take ResolveName into account here, as otherwise file paths will be a disaster!
            Return ESB.Core.Util.ExpandRelativePath(m_LocallyConfiguredServiceRootPath & "\ctx\" & sUrlEncodedServiceName & "\Config\Lcs." & sUrlEncodedServiceName & ".config")
        End If

        Throw New Exception("Error getting LcsPathForRequest: Context = " & sUrlEncodedServiceName & ". ResolveName =" & sUrlEncodedResolveName)

    End Function
    Private Function MakeValidFileName(ByVal sFileName As String) As String
        Return System.Web.HttpUtility.UrlEncode(sFileName)
    End Function

#Region "Helper Methods"
    Public Sub init()
        If Not m_bInit Then
            m_LocallyConfiguredServiceRootPath = System.Configuration.ConfigurationManager.AppSettings(STR_ESBInjectLocalConfigurationHandlerAssembliesRootPath)
            m_bInit = True
        End If
    End Sub
#End Region
End Class
